import json
import logging

import requests

from . import settings

logger = logging.getLogger('django')

AUTH_HEADERS = {
    'AUTHORIZATION': 'Token {0}'.format(settings.CAS_TOKEN),
    'SOURCE-ID': settings.CAS_SOURCE_ID,
}
BASE_URL = settings.CAS_BASE_URL


def call(endpoint, method='GET', payload=None):
    fn = requests.get
    if method == 'POST':
        fn = requests.post
    elif method == 'PUT':
        fn = requests.put
    elif method == 'DELETE':
        fn = requests.delete

    url = '{base_url}/{endpoint}/.json'.format(
        base_url=BASE_URL, endpoint=endpoint)

    request = fn(url, headers=AUTH_HEADERS, json=payload)

    logger.debug('method={0}'.format(method))
    logger.debug('url={0}'.format(url))
    logger.debug('headers={0}'.format(AUTH_HEADERS))
    logger.debug('payload={0}'.format(payload))
    logger.debug('json={0}'.format(json.dumps(payload)))

    _raise_if_4xx_or_5xx_but_not_404(request)

    json_data = {}
    try:
        json_data = request.json()
    except ValueError:
        # Requests chokes on empty body
        pass

    return request.status_code, json_data


def _raise_if_4xx_or_5xx_but_not_404(request):
    try:
        logger.debug('response={0}'.format(request.json()))
    except ValueError:
        # requests chokes on empty json body
        pass
    if not request.status_code == 404:
        request.raise_for_status()



import json

from django.test import TestCase
import pytest
import requests
import responses

from ... import auth_api_client


class TestApiClient(TestCase):

    @responses.activate
    def test_get(self):
        sample_data = {'some': 'data'}
        responses.add(
            responses.GET,
            'https://auth.kagiso.io/api/v1/test_endpoint/.json',
            body=json.dumps(sample_data),
            status=200,
        )

        status, data = auth_api_client.call('test_endpoint')

        assert len(responses.calls) == 1
        assert responses.calls[
            0].request.url == 'https://auth.kagiso.io/api/v1/test_endpoint/.json'  # noqa
        assert status == 200
        assert data == sample_data

    @responses.activate
    def test_post(self):
        sample_data = {'some': 'data'}
        responses.add(
            responses.POST,
            'https://auth.kagiso.io/api/v1/test_endpoint/.json',
            body=json.dumps(sample_data),
            status=201,
        )

        status, data = auth_api_client.call(
            'test_endpoint', 'POST', payload=sample_data)

        assert len(responses.calls) == 1
        assert responses.calls[
            0].request.url == 'https://auth.kagiso.io/api/v1/test_endpoint/.json'  # noqa
        assert status == 201
        assert data == sample_data

    @responses.activate
    def test_put(self):
        sample_data = {'some': 'data'}
        responses.add(
            responses.PUT,
            'https://auth.kagiso.io/api/v1/test_endpoint/1/.json',
            body=json.dumps(sample_data),
            status=200,
        )

        status, data = auth_api_client.call(
            'test_endpoint/1',
            'PUT',
            sample_data
        )

        assert len(responses.calls) == 1
        assert responses.calls[
            0].request.url == 'https://auth.kagiso.io/api/v1/test_endpoint/1/.json'  # noqa
        assert status == 200
        assert data == sample_data

    @responses.activate
    def test_delete(self):
        responses.add(
            responses.DELETE,
            'https://auth.kagiso.io/api/v1/test_endpoint/1/.json',
            body=json.dumps({}),
            status=204,
        )

        status, data = auth_api_client.call('test_endpoint/1', method='DELETE')

        assert len(responses.calls) == 1
        assert responses.calls[
            0].request.url == 'https://auth.kagiso.io/api/v1/test_endpoint/1/.json'  # noqa
        assert data == {}
        assert status == 204

    @responses.activate
    def test_4xx_raises_if_not_404(self):
        responses.add(
            responses.GET,
            'https://auth.kagiso.io/api/v1/test_endpoint/1/.json',
            body=json.dumps({}),
            status=403,
        )

        with pytest.raises(requests.exceptions.HTTPError):
            auth_api_client.call('test_endpoint/1', method='GET')

    @responses.activate
    def test_404_does_not_raise(self):
        responses.add(
            responses.GET,
            'https://auth.kagiso.io/api/v1/test_endpoint/1/.json',
            body=json.dumps({}),
            status=404,
        )

        auth_api_client.call('test_endpoint/1', method='GET')

    @responses.activate
    def test_5xx_raises(self):
        responses.add(
            responses.GET,
            'https://auth.kagiso.io/api/v1/test_endpoint/1/.json',
            body=json.dumps({}),
            status=500,
        )

        with pytest.raises(requests.exceptions.HTTPError):
            auth_api_client.call('test_endpoint/1', method='GET')



from django.contrib.auth.backends import ModelBackend

from . import auth_api_client
from .models import KagisoUser


class KagisoBackend(ModelBackend):

    def authenticate(self, email, password, **kwargs):
        user = KagisoUser.objects.filter(email=email).first()

        if not user:
            return

        payload = {
            'email': email,
            'password': password,
        }

        status, data = auth_api_client.call('sessions', 'POST', payload)

        if status == 200:
            return user



from django.test import TestCase
import responses

from . import mocks
from ...backends import KagisoBackend
from ...models import KagisoUser


class KagisoBackendTest(TestCase):

    @responses.activate
    def test_authenticate_valid_credentials_returns_user(self):
        email = 'test@email.com'
        password = 'random'
        profile = {
            'first_name': 'Fred'
        }
        url, api_data = mocks.mock_out_post_users(
            1,
            email,
            profile=profile
        )
        user = KagisoUser.objects.create_user(
            email, password, profile=profile)
        url = mocks.mock_out_post_sessions(email, password, 200)

        backend = KagisoBackend()
        result = backend.authenticate(email, password)

        assert len(responses.calls) == 2
        assert responses.calls[1].request.url == url

        assert isinstance(result, KagisoUser)
        assert result.id == user.id

    @responses.activate
    def test_authenticate_user_does_not_exist_locally_returns_none(self):
        email = 'test@email.com'
        password = 'random'

        backend = KagisoBackend()
        result = backend.authenticate(email, password)

        assert len(responses.calls) == 0

        assert not result

    @responses.activate
    def test_authenticate_invalid_credentials_returns_none(self):
        email = 'test@email.com'
        password = 'incorrect'
        url, api_data = mocks.mock_out_post_users(1, email)
        KagisoUser.objects.create_user(email, password)
        url = mocks.mock_out_post_sessions(email, password, 404)

        backend = KagisoBackend()
        result = backend.authenticate(email, password)

        assert len(responses.calls) == 2
        assert responses.calls[1].request.url == url

        assert not result
